@@ -2,6 +2,7 @@
 """
 
 import asyncio
+import logging
 
 from denovonear.transcript import Transcript
 from denovonear.gencode import Gencode, Gene
@@ -117,17 +118,21 @@ async def load_gene(ensembl, gene_id):
     """
     
     tx_ids = await get_transcript_ids(ensembl, gene_id)
+    logging.info(f'found tx IDs for {gene_id}: {tx_ids}')
     
     gene = Gene(gene_id)
     for tx_id in tx_ids:
+        logging.info(f'constructing: {tx_id}')
         try:
             tx = await construct_gene_object(ensembl, tx_id)
-        except ValueError:
-            pass
+        except ValueError as e:
+            logging.info(f'failed: {tx_id}')
+            logging.info(e)
+            continue
         gene.add_transcript(tx)
     
     if len(gene.transcripts) == 0:
-        raise IndexError(f"{gene_id} lacks coding transcripts")
+        logging.info(f"{gene_id} lacks coding transcripts")
     
     return gene
 
@@ -183,8 +188,7 @@ def minimise_transcripts(transcripts, de_novos):
     
     # find the transcripts with the greatest length, and the most de novos
     max_length = max( counts[k]["len"] for k in max_ids )
-    tx_ids = [ k.get_name() for k in transcripts if counts[k.get_name()]["len"] == max_length ]
-    max_transcripts = {x: counts[x] for x in counts if x in tx_ids}
+    max_transcripts = {x: counts[x] for x in counts if x in max_ids and counts[x]['len'] == max_length}
     
     # find which de novos occur in the transcript with the most de novos
     best = [x for x in transcripts if x.get_name() == next(iter(max_transcripts))][0]
