@@ -9,28 +9,23 @@
 
 #include "tx.h"
 
-std::unordered_map<char, char> transdict = {
-    {'a', 't'}, {'c', 'g'}, {'g', 'c'}, {'t', 'a'}, {'u', 'a'},
-    {'A', 'T'}, {'C', 'G'}, {'G', 'C'}, {'T', 'A'}, {'U', 'A'},
-    {'N', 'N'}};
-
-std::unordered_map<std::string, std::string> aa_code = {
-    {"AAA", "K"}, {"AAC", "N"}, {"AAG", "K"}, {"AAT", "N"},
-    {"ACA", "T"}, {"ACC", "T"}, {"ACG", "T"}, {"ACT", "T"},
-    {"AGA", "R"}, {"AGC", "S"}, {"AGG", "R"}, {"AGT", "S"},
-    {"ATA", "I"}, {"ATC", "I"}, {"ATG", "M"}, {"ATT", "I"},
-    {"CAA", "Q"}, {"CAC", "H"}, {"CAG", "Q"}, {"CAT", "H"},
-    {"CCA", "P"}, {"CCC", "P"}, {"CCG", "P"}, {"CCT", "P"},
-    {"CGA", "R"}, {"CGC", "R"}, {"CGG", "R"}, {"CGT", "R"},
-    {"CTA", "L"}, {"CTC", "L"}, {"CTG", "L"}, {"CTT", "L"},
-    {"GAA", "E"}, {"GAC", "D"}, {"GAG", "E"}, {"GAT", "D"},
-    {"GCA", "A"}, {"GCC", "A"}, {"GCG", "A"}, {"GCT", "A"},
-    {"GGA", "G"}, {"GGC", "G"}, {"GGG", "G"}, {"GGT", "G"},
-    {"GTA", "V"}, {"GTC", "V"}, {"GTG", "V"}, {"GTT", "V"},
-    {"TAA", "*"}, {"TAC", "Y"}, {"TAG", "*"}, {"TAT", "Y"},
-    {"TCA", "S"}, {"TCC", "S"}, {"TCG", "S"}, {"TCT", "S"},
-    {"TGA", "*"}, {"TGC", "C"}, {"TGG", "W"}, {"TGT", "C"},
-    {"TTA", "L"}, {"TTC", "F"}, {"TTG", "L"}, {"TTT", "F"}};
+std::unordered_map<std::string, char> aa_code = {
+    {"AAA", 'K'}, {"AAC", 'N'}, {"AAG", 'K'}, {"AAT", 'N'},
+    {"ACA", 'T'}, {"ACC", 'T'}, {"ACG", 'T'}, {"ACT", 'T'},
+    {"AGA", 'R'}, {"AGC", 'S'}, {"AGG", 'R'}, {"AGT", 'S'},
+    {"ATA", 'I'}, {"ATC", 'I'}, {"ATG", 'M'}, {"ATT", 'I'},
+    {"CAA", 'Q'}, {"CAC", 'H'}, {"CAG", 'Q'}, {"CAT", 'H'},
+    {"CCA", 'P'}, {"CCC", 'P'}, {"CCG", 'P'}, {"CCT", 'P'},
+    {"CGA", 'R'}, {"CGC", 'R'}, {"CGG", 'R'}, {"CGT", 'R'},
+    {"CTA", 'L'}, {"CTC", 'L'}, {"CTG", 'L'}, {"CTT", 'L'},
+    {"GAA", 'E'}, {"GAC", 'D'}, {"GAG", 'E'}, {"GAT", 'D'},
+    {"GCA", 'A'}, {"GCC", 'A'}, {"GCG", 'A'}, {"GCT", 'A'},
+    {"GGA", 'G'}, {"GGC", 'G'}, {"GGG", 'G'}, {"GGT", 'G'},
+    {"GTA", 'V'}, {"GTC", 'V'}, {"GTG", 'V'}, {"GTT", 'V'},
+    {"TAA", '*'}, {"TAC", 'Y'}, {"TAG", '*'}, {"TAT", 'Y'},
+    {"TCA", 'S'}, {"TCC", 'S'}, {"TCG", 'S'}, {"TCT", 'S'},
+    {"TGA", '*'}, {"TGC", 'C'}, {"TGG", 'W'}, {"TGT", 'C'},
+    {"TTA", 'L'}, {"TTC", 'F'}, {"TTG", 'L'}, {"TTT", 'F'}};
 
 // Constructor for Tx class
 Tx::Tx(std::string transcript_id, std::string chromosome,
@@ -95,6 +90,11 @@ void Tx::set_cds(std::vector<std::vector<int>> cds_ranges) {
         cds.push_back(region);
     }
     
+    if (cds.size() == 0) {
+        // no subsequent adjustments for transcripts without CDS
+        return;
+    }
+    
     cds_min = cds[0].start;
     cds_max = cds.back().end;
     
@@ -187,6 +187,14 @@ int Tx::get_cds_end() {
 // @param position integer chromosome position e.g. 10000000
 bool Tx::is_exonic(int pos) {
     Region exon = get_closest_exon(pos);
+    return is_exonic(pos, exon);
+}
+
+// check if a position lies within an exon
+//
+// @param position integer chromosome position e.g. 10000000
+// @param exon Region for an exon
+bool Tx::is_exonic(int pos, Region exon) {
     return (pos >= exon.start) && (pos <= exon.end);
 }
 
@@ -254,8 +262,8 @@ bool Tx::in_coding_region(int pos) {
 }
 
 // shift an intronic site to the nearest exon boundary (but track the offset)
-CDS_coords Tx::to_closest_exon(int pos) {
-    Region exon = get_closest_exon(pos);
+CDS_coords Tx::to_closest_exon(int pos, Region exon) {
+    // Region exon = get_closest_exon(pos);
     int site;
     if (std::abs(exon.start - pos) < std::abs(exon.end - pos)) {
         site = exon.start;
@@ -277,16 +285,16 @@ CDS_coords Tx::to_closest_exon(int pos) {
 CDS_coords Tx::get_coding_distance(int pos) {
     int cds_start = get_cds_start();
     
+    int first = closest_exon_num(cds_start);
+    int alt = closest_exon_num(pos);
+
     int offset = 0;
-    if ( !is_exonic(pos) ) {
-        CDS_coords site = to_closest_exon(pos);
+    if ( !is_exonic(pos, exons[alt]) ) {
+        CDS_coords site = to_closest_exon(pos, exons[alt]);
         offset = site.offset;
         pos = site.position;
     }
     
-    int first = closest_exon_num(cds_start);
-    int alt = closest_exon_num(pos);
-    
     bool fwd = get_strand() == '+';
     bool after = alt > first;
     bool positive = (fwd & after) | (!fwd & !after);
@@ -395,12 +403,6 @@ void Tx::add_cds_sequence(std::string cds_dna) {
 //    then the DNA sequence will be for the - strand, so we need to
 //    reorient the DNA to the + strand.
 void Tx::add_genomic_sequence(std::string gdna, int offset=0) {
-    if ( cds.size() == 0 ) {
-        std::string msg = "You need to add CDS coordinates before adding"
-            "genomic sequence!";
-        throw std::invalid_argument(msg);
-    }
-    
     char fwd = '+';
     if (get_strand() != fwd) {
         // orient the DNA sequence to the + strand.
@@ -493,6 +495,11 @@ void Tx::_fix_cds_length() {
 
 // reverse complement a DNA or RNA sequence
 std::string Tx::reverse_complement(std::string seq) {
+    static char transdict[128];
+    transdict['a'] = 't'; transdict['c'] = 'g'; transdict['g'] = 'c'; 
+    transdict['t'] = 'a'; transdict['u'] = 'a'; transdict['A'] = 'T'; 
+    transdict['C'] = 'G'; transdict['G'] = 'C'; transdict['T'] = 'A'; 
+    transdict['U'] = 'A'; transdict['N'] = 'N';
     std::reverse(seq.begin(), seq.end());
     std::string complement;
     complement.resize(seq.size());
@@ -550,6 +557,20 @@ std::string Tx::get_codon_sequence(int codon) {
     return cds_sequence.substr(codon * 3, 3);
 }
 
+// translate a single codon to amino acid
+// @param seq codon sequence
+// @returns translated amino acid sequence as single char
+char Tx::translate_codon(std::string codon) {
+    if (codon.size() != 3) {
+        throw std::invalid_argument("translate_codon() needs a 3 bp sequence");
+    }
+    if (aa_code.count(codon) == 0) {
+        std::string msg = "cannot translate codon: " + codon;
+        throw std::invalid_argument(msg);
+    }
+    return aa_code[codon];
+}
+
 // translate a DNA codon to a single character amino acid
 //
 // @param seq codon sequence, or longer DNA sequences.
@@ -558,21 +579,12 @@ std::string Tx::translate(std::string seq) {
     std::transform(seq.begin(), seq.end(), seq.begin(), ::toupper);
     
     std::string protein;
-    int n = 3;
-    int len = seq.size();
-    std::string codon;
+    protein.resize(seq.size() / 3);
     
-    for ( int i=0; i < len; i=i+n ) {
-        codon = seq.substr(i, n);
-        
-        if (aa_code.count(codon) == 0) {
-            std::string msg = "cannot translate codon: " + codon;
-            throw std::invalid_argument( msg );
-        }
-        
-        protein += aa_code[codon];
+    for (uint i=0, j=0; i<seq.size(); i=i+3, j++) {
+        protein[j] = translate_codon(seq.substr(i, 3));
     }
-    
+
     return protein;
 }
 
@@ -594,13 +606,13 @@ Codon Tx::get_codon_info(int bp) {
     int codon_number = -9999999;
     int intra_codon = -1;
     std::string codon_seq = "";
-    std::string initial_aa = "";
+    char initial_aa='0';
     
     if (in_coding) {
         codon_number = get_codon_number_for_cds_position(site.position);
         intra_codon = get_position_within_codon(site.position);
         codon_seq = get_codon_sequence(codon_number);
-        initial_aa = translate(codon_seq);
+        initial_aa = translate_codon(codon_seq);
     }
     
     return Codon {site.position, codon_seq, intra_codon, codon_number,
@@ -709,12 +721,12 @@ std::string Tx::coding_cq(int start, int end, std::string alt) {
     }
     // TODO: figure out initial and mutated amino acids
     Codon codon = get_codon_info(start);
-    std::string initial_aa = codon.initial_aa;
-    codon.codon_seq.replace(codon.intra_codon, 1, alt);
-    std::string mutated_aa = translate(codon.codon_seq);
+    char initial_aa = codon.initial_aa;
+    codon.codon_seq[1] = alt[0];
+    char mutated_aa = translate_codon(codon.codon_seq);
     
-    if (initial_aa != "*" && mutated_aa == "*") { return "stop_gained"; }
-    if (initial_aa == "*" && mutated_aa != "*") { return "stop_lost"; }
+    if (initial_aa != '*' && mutated_aa == '*') { return "stop_gained"; }
+    if (initial_aa == '*' && mutated_aa != '*') { return "stop_lost"; }
     
     if (initial_aa != mutated_aa) {
         if (codon.codon_number == 0) {
