@@ -68,11 +68,11 @@ class TestGencode(unittest.TestCase):
                 'chr1\tHAVANA\texon\t10\t20\t.\t-\t.\ttranscript_id "ENST_A" gene_name "TEST1"; transcript_type "protein_coding"\n' \
                 'chr1\tHAVANA\tCDS\t15\t20\t.\t-\t.\ttranscript_id "ENST_A" gene_name "TEST1"; transcript_type "protein_coding"\n' \
                 'chr1\tHAVANA\tgene\t100\t110\t.\t-\t.\tgene_name "TEST2";\n' \
-                'chr1\tHAVANA\ttranscript\t100\t100\t.\t-\t.\ttranscript_id "ENST_B";gene_name "TEST2"; transcript_type "protein_coding"; tag "appris_principal_1";\n' \
+                'chr1\tHAVANA\ttranscript\t100\t110\t.\t-\t.\ttranscript_id "ENST_B";gene_name "TEST2"; transcript_type "protein_coding"; tag "appris_principal_1";\n' \
                 'chr1\tHAVANA\texon\t100\t110\t.\t-\t.\ttranscript_id "ENST_B" gene_name "TEST2"; transcript_type "protein_coding"\n' \
-                'chr1\tHAVANA\tCDS\t115\t1100\t.\t-\t.\ttranscript_id "ENST_B" gene_name "TEST2"; transcript_type "protein_coding"\n'\
+                'chr1\tHAVANA\tCDS\t105\t110\t.\t-\t.\ttranscript_id "ENST_B" gene_name "TEST2"; transcript_type "protein_coding"\n'\
                 'chr2\tHAVANA\tgene\t100\t110\t.\t-\t.\tgene_name "TEST3";\n' \
-                'chr2\tHAVANA\ttranscript\t100\t100\t.\t-\t.\ttranscript_id "ENST_C";gene_name "TEST3"; transcript_type "protein_coding"; tag "appris_principal_1";\n' \
+                'chr2\tHAVANA\ttranscript\t100\t110\t.\t-\t.\ttranscript_id "ENST_C";gene_name "TEST3"; transcript_type "protein_coding"; tag "appris_principal_1";\n' \
                 'chr2\tHAVANA\texon\t100\t110\t.\t-\t.\ttranscript_id "ENST_C" gene_name "TEST3"; transcript_type "protein_coding"\n' \
                 'chr2\tHAVANA\tCDS\t105\t110\t.\t-\t.\ttranscript_id "ENST_C" gene_name "TEST3"; transcript_type "protein_coding"\n'
         
@@ -81,22 +81,35 @@ class TestGencode(unittest.TestCase):
             make_fasta(fasta.name, ['chr1', 'chr2'])
             gencode = Gencode(temp.name, fasta.name)
         
-        # check different region selections, just by examining the number of 
-        # genes, and their HGNC symbols. In other parts we check gene setup, so
-        # we don't need to repeat that here
-        in_region = gencode.in_region('chr1', 0, 5)
-        self.assertEqual(len(in_region), 0)
-        in_region = gencode.in_region('chr1', 10, 30)
-        self.assertEqual(len(in_region), 1)
-        self.assertEqual(in_region[0].symbol, "TEST1")
-        in_region = gencode.in_region('chr1', 10, 2000)
-        self.assertEqual(len(in_region), 2)
-        self.assertEqual(set(x.symbol for x in in_region), set(["TEST1", "TEST2"]))
+        genes = gencode.in_region('chr1', 5, 6)  # shouldn't get anything
+        self.assertEqual(genes, [])
+        genes = gencode.in_region('chr1', 5, 15)  # should get TEST1 (via start)
+        self.assertEqual(set(x.symbol for x in genes), set(["TEST1"]))
+        genes = gencode.in_region('chr1', 5, 25)  # should get TEST1 (via start, end)
+        self.assertEqual(set(x.symbol for x in genes), set(["TEST1"]))
+        genes = gencode.in_region('chr1', 15, 25)  # should get TEST1 (via end)
+        self.assertEqual(set(x.symbol for x in genes), set(["TEST1"]))
+        genes = gencode.in_region('chr1', 12, 13) # should get TEST1 (via encapsulate)
+        self.assertEqual(set(x.symbol for x in genes), set(["TEST1"]))
+        genes = gencode.in_region('chr1', 5, 105) # should get TEST1 (via start, end), TEST2 (via start)
+        self.assertEqual(set(x.symbol for x in genes), set(["TEST1", "TEST2"]))
+        genes = gencode.in_region('chr1', 5, 115) # should get TEST1 (via start, end), TEST2 (via start, end)
+        self.assertEqual(set(x.symbol for x in genes), set(["TEST1", "TEST2"]))
+        genes = gencode.in_region('chr1', 15, 105) # should get TEST1 (via end), TEST2 (via start)
+        self.assertEqual(set(x.symbol for x in genes), set(["TEST1", "TEST2"]))
+        genes = gencode.in_region('chr1', 25, 105)  # should get TEST2 (via start)
+        self.assertEqual(set(x.symbol for x in genes), set(["TEST2"]))
+        genes = gencode.in_region('chr1', 102, 105) # should get TEST2 (via encapsulate)
+        self.assertEqual(set(x.symbol for x in genes), set(["TEST2"]))
+        genes = gencode.in_region('chr1', 120, 150) # shouldn't get anything
+        self.assertEqual(genes, [])
         
         # no overlapping genes right up to the border of a gene
-        in_region = gencode.in_region('chr1', 0, 10)
+        in_region = gencode.in_region('chr1', 0, 9)
         self.assertEqual(len(in_region), 0)
-        in_region = gencode.in_region('chr1', 20, 30)
+        in_region = gencode.in_region('chr1', 9, 9) # check a single base before
+        self.assertEqual(len(in_region), 0)
+        in_region = gencode.in_region('chr1', 21, 30)
         self.assertEqual(len(in_region), 0)
         
         # we find matches if even a single base of the gene is in the region
@@ -112,31 +125,32 @@ class TestGencode(unittest.TestCase):
                 'chr1\tHAVANA\texon\t10\t20\t.\t-\t.\ttranscript_id "ENST_A" gene_name "TEST1"; transcript_type "protein_coding"\n',
                 'chr1\tHAVANA\tCDS\t15\t20\t.\t-\t.\ttranscript_id "ENST_A" gene_name "TEST1"; transcript_type "protein_coding"\n',
                 'chr1\tHAVANA\tgene\t100\t110\t.\t-\t.\tgene_name "TEST2";\n',
-                'chr1\tHAVANA\ttranscript\t100\t100\t.\t-\t.\ttranscript_id "ENST_B"; gene_name "TEST2"; transcript_type "protein_coding"; tag "appris_principal_1";\n',
+                'chr1\tHAVANA\ttranscript\t100\t110\t.\t-\t.\ttranscript_id "ENST_B"; gene_name "TEST2"; transcript_type "protein_coding"; tag "appris_principal_1";\n',
                 'chr1\tHAVANA\texon\t100\t110\t.\t-\t.\ttranscript_id "ENST_B" gene_name "TEST2"; transcript_type "protein_coding"\n',
-                'chr1\tHAVANA\tCDS\t115\t1100\t.\t-\t.\ttranscript_id "ENST_B" gene_name "TEST2"; transcript_type "protein_coding"\n',
+                'chr1\tHAVANA\tCDS\t105\t110\t.\t-\t.\ttranscript_id "ENST_B" gene_name "TEST2"; transcript_type "protein_coding"\n',
                 'chr2\tHAVANA\tgene\t100\t110\t.\t-\t.\tgene_name "TEST3";\n',
-                'chr2\tHAVANA\ttranscript\t100\t100\t.\t-\t.\ttranscript_id "ENST_C"; gene_name "TEST3"; transcript_type "protein_coding"; tag "appris_principal_1";\n',
+                'chr2\tHAVANA\ttranscript\t100\t110\t.\t-\t.\ttranscript_id "ENST_C"; gene_name "TEST3"; transcript_type "protein_coding"; tag "appris_principal_1";\n',
                 'chr2\tHAVANA\texon\t100\t110\t.\t-\t.\ttranscript_id "ENST_C" gene_name "TEST3"; transcript_type "protein_coding"\n',
                 'chr2\tHAVANA\tCDS\t105\t110\t.\t-\t.\ttranscript_id "ENST_C" gene_name "TEST3"; transcript_type "protein_coding"\n']
         with tempfile.NamedTemporaryFile() as temp, tempfile.NamedTemporaryFile() as fasta:
             write_gtf(temp.name, lines)
             make_fasta(fasta.name, ['chr1', 'chr2'])
             gencode = Gencode(temp.name, fasta.name)
+            
+            self.assertEqual(gencode.nearest('chr1', 5).symbol, 'TEST1')
+            self.assertEqual(gencode.nearest('chr1', 10).symbol, 'TEST1')
+            self.assertEqual(gencode.nearest('chr1', 11).symbol, 'TEST1')
+            self.assertEqual(gencode.nearest('chr1', 21).symbol, 'TEST1')
+            self.assertEqual(gencode.nearest('chr1', 80).symbol, 'TEST2')
+            self.assertEqual(gencode.nearest('chr1', 105).symbol, 'TEST2')
+            self.assertEqual(gencode.nearest('chr1', 2000).symbol, 'TEST2')
         
-        self.assertEqual(gencode.nearest('chr1', 5).symbol, 'TEST1')
-        self.assertEqual(gencode.nearest('chr1', 10).symbol, 'TEST1')
-        self.assertEqual(gencode.nearest('chr1', 11).symbol, 'TEST1')
-        self.assertEqual(gencode.nearest('chr1', 21).symbol, 'TEST1')
-        self.assertEqual(gencode.nearest('chr1', 80).symbol, 'TEST2')
-        self.assertEqual(gencode.nearest('chr1', 2000).symbol, 'TEST2')
-        
-        # and look for genes on the final chrom, both before and after the final gene
-        self.assertEqual(gencode.nearest('chr2', 0).symbol, 'TEST3')
-        self.assertEqual(gencode.nearest('chr2', 2000).symbol, 'TEST3')
-        
-        with self.assertRaises(ValueError):
-            gencode.nearest('chrZZZ', 2000)
+            # and look for genes on the final chrom, both before and after the final gene
+            self.assertEqual(gencode.nearest('chr2', 0).symbol, 'TEST3')
+            self.assertEqual(gencode.nearest('chr2', 2000).symbol, 'TEST3')
+            
+            with self.assertRaises(ValueError):
+                gencode.nearest('chrZZZ', 2000)
     
     def test_gencode_canonical(self):
         ''' test we find the correct canonical transcript
@@ -147,7 +161,7 @@ class TestGencode(unittest.TestCase):
                 'chr1\tHAVANA\texon\t20\t30\t.\t-\t.\ttranscript_id "ENST_A" gene_name "TEST1"; transcript_type "protein_coding"\n',
                 'chr1\tHAVANA\tCDS\t25\t30\t.\t-\t.\ttranscript_id "ENST_A" gene_name "TEST1"; transcript_type "protein_coding"\n',
                 'chr1\tHAVANA\tgene\t100\t110\t.\t-\t.\tgene_name "TEST1";\n',
-                'chr1\tHAVANA\ttranscript\t100\t100\t.\t-\t.\ttranscript_id "ENST_B"; gene_name "TEST1"; transcript_type "protein_coding";\n',
+                'chr1\tHAVANA\ttranscript\t100\t110\t.\t-\t.\ttranscript_id "ENST_B"; gene_name "TEST1"; transcript_type "protein_coding";\n',
                 'chr1\tHAVANA\texon\t100\t110\t.\t-\t.\ttranscript_id "ENST_B" gene_name "TEST1"; transcript_type "protein_coding"\n',
                 'chr1\tHAVANA\tCDS\t110\t100\t.\t-\t.\ttranscript_id "ENST_B" gene_name "TEST1"; transcript_type "protein_coding"\n',
         ]
